/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The conversation view abstraction layer.
 *
 * @class ? @extends glUIView
 */
var glConversationView = glUIView.extend({
    // Properties {
    /**
     * The {@link glActionsHeader} object for the page
     * @property {glActionsHeader} ?
     */
    actionsHeader: null,
    
    /** @property {Element} ? The header container in Conversation */
    header: null,
    
    /** @property {String} ? The thread ID of this conversation */
    threadId: null,
    
    /** @property {private glThread} ? Cached copy of thread associated with this conversation */
    _thread: null,
	
	
	
	
	
    //}

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {glGmail} gmail - The glGmail object for the page
     */
    connect: function(gmail) {
        glLogger.log('Connecting glConversationView object to view', 'glConversationView');
        this.base(gmail);

        this.actionsHeader = gmail.getUIObject('actionsHeader');
        
        var elRoot = this._glGmail.page.getViewRoot();
        var doc = gmail.canvasDocument;
        var result = doc.evaluate(".//h1[@class='ha']", elRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
	if( !result.singleNodeValue ) result = doc.evaluate(".//h1[@class='YfMhcb']", elRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null);
        this.header = result.singleNodeValue;


	var m = gmail.canvasDocument.defaultView.window.top.location.href.match(/\/([^\/]+?)$/);
	if( m ){
	    if( m[1]=="#sent" ){
		this.threadId = gmail.environment.lastSentThreadId;
	    }else{
		this.threadId = m[1]? m[1] : null;
	    }
	}
	
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting glConversationView from view', 'glConversationView');
        
        //glLogger.warn('unloading glConversationView', 'glConversationView.disconnect');
        if( !this._glGmail ) return;    // TODO this function is being called twice on shutdown, need to work out why. 
        
    	this._glGmail.actions.removeEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
        
        this.header = null;
        this.threadId = null;
        this._thread = null;
        this.actionsHeader.disconnect();
        this.actionsHeader = null;
        
        this.base();
    },

    /**
    * Get the 'Newer' element - if it exists - from the navigation section
    * @function {Element} ?
    * @return The 'Newer' element
    */
    getMovePreviousElement: function(){
        var elNav = this._glGmail.canvasDocument.evaluate(".//div[@class=iG']/div[@class='h0']", this.actionsHeader.root, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        if( elNav.firstChild.nodeName=="SPAN" ) return elNav.firstChild; 	// <span><b><b><span>
    },

    /**
    * Get the 'Older' element - if it exists - from the navigation section
    * @function {Element} ?
    * @return The 'Older' element
    */
    getMoveNextElement: function(){
        var elNav = this._glGmail.canvasDocument.evaluate(".//div[@class='iG']/div[@class='h0']", this.actionsHeader.root, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        if( elNav.childNodes.length>0 && elNav.lastChild.nodeName=="SPAN" ) return elNav.lastChild; 	// <span><b><b><span>
    },

    /**
    * Get the thread associated with this view
    * @function {glThread} ?
    */
    getThread: function(){
        if( !this._thread ){
            if( this.threadId ){
                this._thread = glThread.createFromId(this.threadId, this._glGmail);
                this._glGmail.actions.addEventListener(glActionManager.THREAD_CHANGED, this._threadChanged, this);
            }
        }
        return this._thread;
    },
	
	
    /**
     * Callback for the {@link glActionManager.THREAD_CHANGED} event. 
     * @function ?
     * @param {Event} evt - The thread changed event
     */
    _threadChanged: function(evt) {
        // Find instances of the thread
        if( this._thread.threadId==evt.thread ){
            this._thread.updateFromEvent(evt);
        }
    },
	
    
    //}
});
