/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Add Lightbox toggling functionality
 *
 * @class ? @extends glUIWidget
 */
 
var glLightbox = glUIWidget.extend({

	// Properties {
	
	
	
	/**
     	* The container for the entire ShowHider
    	* @property {Element} ?
     	*/
	el: null,
	
	/** @property {glEventsContainer} */
	_eventsContainer: null,
	

	// }
	
	// Methods {
	
	
	/**
	     * Connects the element to the given glGmail object's page. 
     	* @param {glGmail} gmail - The glGmail object for the page
	*/
	connect: function(gmail) {
	        glLogger.log('glLightbox object created', 'glLightbox');
	
		this.base(gmail);
		this._eventsContainer = new glEventsContainer(this);
		
		var doc = gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.className = 'lightbox';
		this.el.innerHTML = '&nbsp;';
		this.el.style.display = 'none';
		doc.body.insertBefore(this.el, doc.body.firstChild);
		
		
	},
	
	disconnect: function(){
		this._eventsContainer.reset();
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild( this.el );
		this.el = null;
		this.base();
	},
	
	show: function(v){
		this.el.style.display = '';
		
		this._eventsContainer.observe(window, "resize", this._position, false );
		this._position();
		
	},
	
	hide: function(){
		this.el.style.display = 'none';
		this._eventsContainer.stopObserving(window, "resize", this._position, false );
	},
	
	_position: function(){

		try{
			var doc = this._glGmail.canvasDocument;
		
			var winDims = glUIWidget.viewport(doc);

			this.el.style.height = winDims[1] + "px";
			this.el.style.width = winDims[0] + "px";

		}catch(e){
			glLogger.warn(e, 'glLightbox._position', this._glGmail);
		}

	},
	

});