/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * Abstraction layer for the GMail search box
 *
 * @class ? @extends glUIWidget
 */
var glSearchBox = glUIWidget.extend({
    // Properties {
    /** @property {Element} ? The search box element */
    searchBox: null,

    /** @property {Element} ? The "Search Mail" button */
    searchMailButton: null,

    /** @property {Element} ? The "tr" element containing the search elements */
    containerRow: null,
    //}

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {glGmail} gmail - The glGmail object for the page
     */
    connect: function(gmail) {
        glLogger.log('Connecting glSearchBox object to view', 'glSearchBox');

        this.base(gmail);

        var doc = gmail.canvasDocument;

        this.searchBox = doc.getElementById(':ra');
        if(!this.searchBox) glLogger.error('Could not find searchBox node', 'glSearchBox');

        this.searchMailButton = doc.getElementById(':r9');
        if(!this.searchMailButton) glLogger.error('Could not find searchMailButton node', 'glSearchBox');

        this.containerRow = this.searchBox.parentNode.parentNode;
        if(!this.containerRow) glLogger.error('Could not find containerRow node', 'glSearchBox');
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting glSearchBox from view', 'glSearchBox');
        this.searchBox = null;
        this.searchMailButton = null;
        this.containerRow = null;

        this.base();
    },

    /**
     * Returns the value of the search box
     * @function {String} ?
     * @return The text in the search box
     */
    getSearch: function() {
        return this.searchBox.value;
    },

    /**
     * Sets the value of the search box to the given value
     * @function ?
     * @param {String} value - The text to put in the search box
     */
    setSearch: function(value) {
        this.searchBox.value = value;
    },

    /**
     * Submits the search
     * @function ?
     */
    submit: function() {
        glUIWidget.simulateMouseEvent(this.searchMailButton, 'mousedown');
        glUIWidget.simulateMouseEvent(this.searchMailButton, 'mouseup');
    }
    //}
});
