/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * GTD plugin for the main Gmail page.
 *
 * @class ?
 */
var gtdMainPagePlugin = glPluginBase.extend({
    // Properties {
    

    /**
    * Contains LabelsData
    * @property {gtdLabelsData} ?
    */
    labelsData: null,

    /** @property {gtdEnvironment} ? Listen to clicks */
    environment: null,
    
    /** @property {gtdThreadContainer} */
    statusThreadsContainer: null,
    
    /** @property {gtdThreadRules} */
    threadRules: null,
    
    /** @property {gtdRecent} */
    recent: null,
    
    /** @property {String} ? 'tl', 'cv', etc. */
    _activeViewType: null,
    
    /**
     * True if baseUIReady has completed
     * @property {Boolean} ?
     */
    _loaded: null,
    
    /**
     * True if VIEW_CHANGE fired becaose baseUIReady was complete
     * @property {Boolean} ?
     */
    _missedFirstViewChange: null,
    
    
    /**
     * A hash of all the UI objects
     * @property {Object} ?
     */
    _ui: null,
    //}

    /**
     * The constructor.
     * @constructor ?
     */
    constructor: function() {
        glLogger.log('gtdMainPagePlugin object created', 'gtdMainPagePlugin');
	this.base();
    },

    /**
     * Load components required for Gmail integration; and kick-start all required components
     * @function {} ?
     * @param {glGmail} gmail - The glGmail object for the page
     */
    loadForGmail: function(gmail){
	
	glLogger.log('loading gtdMainPagePlugin for gmail', 'gtdMainPagePlugin');
	this.base(gmail);
        
	
	// Load files for plugin:
	plugins = this.gmail.plugins;
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdUIObject.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdMainView.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdComposeTabs.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdComposeView.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdComposePersonal.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdConversationView.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdThreadListView.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdThreadListCommands.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdSearchButton.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdLabellingBox.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdLabellingDropdown.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdThreadListContainer.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdLabelsMenu.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdExplorerThreadsViewer.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdExplorerThreadsItem.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdExplorerViewSwitch.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdThreadList.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdExplorerPopup.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdExplorerThreadViewer.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdThread.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdExplorerBundle.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdNavBar.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdConversationAlerts.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdLabelsTree.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/ui/gtdLabelsBox.js");
      
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/gtdThreadContainer.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/gtdLabelsData.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/gtdEnvironment.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/gtdThreadRules.js");
	plugins.loadChromeFile("chrome://gtdinbox/content/js/gtd/gtdRecent.js");
	

	// Load the international properties for the entire gtd plugin
        glI18N.addStringBundle("gtd");



	// Update the preferences if necessary
	var gtdPrefs = this.gmail.accountData.getPref('plugins.gtd.prefs');
	if( gtdPrefs ){
	    // Check for migration to a newer version.
	    var accountData = this.gmail.accountData;
	    var prefsChanged = false;
	    switch(gtdPrefs.version){
		// If curious, see migrate routine in glAccountData for example. Use accountData.setPref rather than gtdPrefs
	    }
	    if( prefsChanged ){
		accountData.savePrefs();
	    }
	}else{
	    // Setting up for first time. Load defaults
	    this.gmail.accountData.setPref('plugins.gtd.prefs', gtdMainPagePlugin.DEFAULT_PREFS);
	    this.gmail.accountData.savePrefs();
	}



        // Initialize UI object container
        this._ui = {};
	
        
        
        // Initiate Gmail events:
	if( this.gmail.page.isUIReady() ){
	    this._baseUIReady();
	}else{
	    this.gmail.page.addEventListener(glPageWrapper.UI_READY, this._baseUIReady, this);
	}
	this.gmail.page.addEventListener(glPageWrapper.VIEW_CHANGED, this._viewChanged, this);
	if( this.gmail.page.hasFiredFirstViewChange() ){
	    // We missed our first load; so fire it manually
	    this._viewChanged();
	}
        
	
    },

    /**
     * Called on page unload, it cleans up everything
     * @function ?
     */
    unload: function() {
	if( this.statusThreadsContainer ){
            this.statusThreadsContainer.preventUnload = false;
            this.statusThreadsContainer.unload();
	}
		
        //this.accountData.shutdown();
        
        this.labelsData.unload();
	this.environment.unload();
		
        for(var element in this._ui) {
            this._ui[element].disconnect();
        }
        
        this.base();
    },


    
    /**
     * Returns a list of the required glGmail components
     * @function {Array} ?
     * @return The required glGmail components
     */
    getRequiredComponents: function() {
        return ['trafficWatcher'];
    },
    

    /**
     * Returns the given UI object after initializing it.
     * For example, if you wanted a gtdMainView object, you would call
     * getUIObject('mainView'), dropping the gtd and turning the 'M' to an
     * 'm'. If the object is cacheable and has already been loaded, it returns
     * the one in the cache.
     * @function {gtdUIObject} ?
     * @param {String} type - The UI object to return
     * @return The initialized UI object
     */
    getUIObject: function(type) {
        glLogger.log('Getting '+type+'...', 'gtdMainPagePlugin');

        var object = null;
        if(!this._ui[type]) {
            // Not in cache so create and cache (or return if not supposed to cache)

            // Add gtd to the begining of the type and raise the first character up
            // mainView -> gtdMainView
            theClass = 'gtd'+type.substr(0,1).toUpperCase()+type.substr(1);

            glLogger.log('Creating object: "'+theClass+'"', 'glGmail');

            try {
                object = new window[theClass]();
            } catch(e) {
                // Use glLogger.error because it gives a stack trace
                glLogger.error("Failure instantiating "+theClass+" ui object:\n"+e.message, 'gtdMainPagePlugin');
            }
    
            object.connect(this);
		
            // Is the ui object cachable - return if not
            if(object.isCacheable()) {
                // Cache the instantiated object
                this._ui[type] = object;
            }
        } else {
            // Get object from cache
            object = this._ui[type];
        }

        return object;
    },

    /**
    * Get a gtdThreadContainer loaded with statuses
    * @function {gtdThreadContainer} ?
    */	
    createStatusThreadsContainer: function(){
	// Create the query string:
	var typeSettings = this.gmail.accountData.getPref('plugins.gtd.prefs.labels.types');
	var status = this.labelsData.getStatusData();
	var statuses = this.labelsData.getLabelsArray("statuses", false, true);
	var orConditions = new glSearchOrConditions();
	var seenStatuses = {};
	
	// Include all core statuses - ensures we have a tight query even if user does not have Status labels created
	for( c in typeSettings.statuses.core ){
	    if( c!="finished" ){
		seenStatuses[status[c]] = true;
		orConditions.addCondition(glSearch.l(status[c]))
	    }
	}
	// Get any unique statuses
	for( var i = 0; i < statuses.length; i++){
		if( statuses[i]!=status.finished && !seenStatuses[statuses[i]] ){
			orConditions.addCondition(glSearch.l(statuses[i]));
		}
	}
	var search = new glSearch(this.gmail);
	search.addCondition(orConditions);
	search.addCondition('-'+glSearch.l(status.finished));
	
	glLogger.log('query string\n' + search.getQuery(), 'gtdMainPagePlugin.getStatusThreadContainer');
	
	// Create the gtdThreadContainer with this query string:
	if( !this.statusThreadsContainer ){
		this.statusThreadsContainer = gtdThreadContainer.createWithSearch(this, search);
		this.statusThreadsContainer.addEventListener(gtdThreadContainer.CHANGED, function(event){
			glLogger.log("\n\nStatus Thread Container change", "gtdMainPagePlugin.getStatusThreadsContainer");
		}, this);
		this.statusThreadsContainer.search();
		this.statusThreadsContainer.preventUnload = true;
	}else{
		this.statusThreadsContainer.update(search);
	}
	

	return this.statusThreadsContainer;
    },

    /**
     * Finishes all the loading that GTDGmail needs. A callback to the
     * {@link glPageWrapper.UI_READY} event.
     * @function ?
     * @param {Event} evt - The UI_READY event
     */
    _baseUIReady: function(evt) {
	// Remove event listener
	this.gmail.page.removeEventListener(glPageWrapper.UI_READY, this._baseUIReady, this);

	this.gmail.fb = this.gmail.canvasDocument.defaultView.wrappedJSObject.top.console;

	// Add the CSS // TODO make part of plugin structure
	glGmail.addCSS(this.gmail.canvasDocument, "chrome://gtdinbox/skin/glGmail.css");
	glGmail.addCSS(this.gmail.canvasDocument, "chrome://gtdinbox/skin/gtd/c/reset.css");
	glGmail.addCSS(this.gmail.canvasDocument, "chrome://gtdinbox/skin/gtd/c/main.css");

	// Load the Labels box:
	this.labelsData = new gtdLabelsData(this);
	
	// Load the environment box:
	this.environment = new gtdEnvironment(this);
	
	// Load the thread rules
	this.threadRules = new gtdThreadRules(this);
	
	// Load the recent
	this.recent = new gtdRecent(this);
	
	// TODO either in the gtdThreadContainer, or below, add listeners that refresh the statusThreadContainer when labels change.

	// Load the threadContainer for statuses
	this.createStatusThreadsContainer();
	this.labelsData.addEventListener(gtdLabelsData.LABEL_CHANGED, glBase.bind(this, function(event){
		// Label changed. 
		// If add a label, it can be added to a thread upon creation. If that is a status label, it may mean we need to add a new thread.
		// If rename a label, it means our search query can be out of date.
		// If we remove a label, we may no longer wish to include threads that had that label.
		// So, in all cases, we need to update the query to use latest statuses; then update.
		this.createStatusThreadsContainer();
	}), this);

	// Load gtdMainView, which loads all the rest of the GTD widgets
	var mainView = this.getUIObject('mainView');
	        
        // Update load status and check flow:
        this._loaded = true;
        if( this._missedFirstViewChange ){
            // Fired a VIEW_CHANGE while we were loading, which will have been denied. Let it run now.
            this._viewChanged();
        }
        
    },

    /**
     * Triggered by the {@link glPageWrapper.VIEW_CHANGED} event, it calls
     * getCurrentViewUIObject to make sure it gets connected. This is the wrong
     * behavior, but necessary for now.
     * @param {Event} evt - The VIEW_CHANGED event
     */
    _viewChanged: function(evt) {
        
        // Update load status and check flow:
        if( !this._loaded ){
            this._missedFirstViewChange = false;
            return; // Cannot run until base is loaded. _baseUIReady will ensure this runs when it has loaded.
        }
        
        
        glLogger.log("start view changed", "gtdMainPagePlugin");
        this._activeViewType = this.gmail.page.getActiveViewType();
        
        var view = this.gmail.getCurrentViewUIObject();	// Ensure it gets connected
           
        
        // In the cache, so disconnect
        if(this._ui['currentViewObject']){
                this._ui['currentViewObject'].disconnect();
                this._ui['currentViewObject'] = null;
        }


        // Get the UI object type for a call to getUIObject
        var uiObject = '';
        switch(this.gmail.page.getActiveViewType()) {
            case 'tl': uiObject = 'threadListView'; break;
            case 'cv': uiObject = 'conversationView'; break;
            case 'co': uiObject = 'composeView'; break;
        }

        if(uiObject) {
            // Use getUIObject so extending classes can override the view objects without overriding this method
            this._ui['currentViewObject'] = this.getUIObject(uiObject);
        }
        

        glLogger.log("end view changed", "gtdMainPagePlugin");
    },
    
    
    /**
     * Returns the UI object for the current view
     * @function {glUIView} ?
     * @return The view UI object
     */
    getCurrentViewUIObject: function() {
        // In the cache, so return
        if(typeof this._ui['currentViewObject'] != 'undefined') {
            return this._ui['currentViewObject'];
        }
    },
    
    //}
},{
    
    // Static Properties {
    /**
     * The default properties object
     * @property {static Object} ?
     */    
    DEFAULT_PREFS: {
	version: '1.0.0',
	labels: {
            types: {
                statuses: { prefix: 'S/', core: {today: 'Today', next_action: 'Next Action', action: 'Action', waiting_on: 'Waiting On', some_day: 'Some Day', finished: 'Finished'} },
                projects: { prefix: 'P/' },
                contexts: { prefix: 'C/' },
                references: { prefix: 'R/' },
                old: { prefix: 'Old/' },
                misc: { prefix: '' },
            },
        },
        components: {
	    navigation: {
		dashboard: { enabled: true },
		status_links: { enabled: true },
		label_box: {
		    expanded: ['statuses', 'projects', 'contexts', 'misc']
		},
	    },
	    thread_list: {
		preview_button: { enabled:true },
		trash_button: { enabled:true },
	    },
	    conversation: {
		labelling_box: { enabled:true },
		reply_suggest: { enabled:true },
	    },
	    search: {
		search_button: { enabled: true },
	    },
	    compose: {
		compose_personal: { enabled: true },
		compose_personal_tabs: {enabled: true},
	    },
        },
        general: {
        	centre_popup: true,
        },
    },
    
    // }
});

