/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The conversation view abstraction layer.
 *
 * @class ? @extends glUIView
 */
var gtdConversationView = gtdUIObject.extend({
    // Properties {
    

    /** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
    _gtd: null,

    
    /**
    * The {@link gtdLabellingBox} object
    * @property {gtdLabellingBox} ?
    */
    _conversationLabels: null,
    
    /**
    * The {@link gtdConversationAlerts} object
    * @property {gtdConversationAlerts} ?
    */
    _conversationAlerts: null,
	
    //}

    // Methods {
    /**
     * Connects the element to the given glGmail object's page. Called automatically
     * by {@link glGmail.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        glLogger.log('Connecting gtdConversationView object to view', 'gtdConversationView');
        this.base(gtd);
        
        this._gtd = gtd;
        
                    
        var doc = gtd.gmail.canvasDocument;
        var viewRoot = gtd.gmail.page.getViewRoot();
        
        
        // Add labels:
    
            
        var elH1 = doc.evaluate(".//h1[@class='ha']", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
        if( !elH1 ) elH1 = doc.evaluate(".//h1[@class='YfMhcb']", viewRoot, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;


        var conversationView = gtd.gmail.getCurrentViewUIObject();
        var thread = conversationView.getThread();
        
        if( thread ){
            
            if( gtd.gmail.accountData.getPref('plugins.gtd.prefs.components.conversation.labelling_box.enabled') ){
                this._conversationLabels = gtd.getUIObject("labellingBox");
                this._conversationLabels.setServerActionType(glActionManager.USE_DOM);
                glUIWidget.addClassName(this._conversationLabels.el, 'conversation-labels');
                var elSpan = this._conversationLabels.el.ownerDocument.createElement('SPAN');
                elSpan.className = 'gtdi-clearfix';
                this._conversationLabels.el.appendChild( elSpan );
                elH1.parentNode.appendChild( this._conversationLabels.el );
            }
            
            if( thread.isLoading() ){
                thread.addEventListener(glThread.LOADED, this._threadLoaded, this)
            }else{
                this._threadLoaded();
            }
            
            // Add listener for alert boxes
            if( gtd.gmail.accountData.getPref('plugins.gtd.prefs.components.conversation.reply_suggest.enabled') ){
                this._conversationAlerts = gtd.getUIObject("conversationAlerts");
            }
            
            
        }else{
            glLogger.warn('could not load thread in conversation (glConversation threadid: ' + conversationView.threadId  + ')', 'gtdConversationView.connect');
        }
        
        // Add listener for Archive
        this._gtd.gmail.actions.addEventListener(glActionManager.ARCHIVE.eventName, this._threadArchived, this);
    },


    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting gtdConversationView from view', 'gtdConversationView');

        
        if( this._conversationLabels ){
            this._conversationLabels.disconnect();
            this._conversationLabels = null;
        }
        
        if( this._conversationAlerts ){
            this._conversationAlerts.disconnect();
            this._conversationAlerts = null;
        }

        this.base();
    },

    /**
    * Notification that a thread was fully loaded (inc messages)
    * Load thread in labelling box
    * @function {} ?
    */
    _threadLoaded: function(evt){
        var conversationView = this._gtd.gmail.getCurrentViewUIObject();
        var thread = conversationView.getThread();
        
        if( this._gtd.gmail.accountData.getPref('plugins.gtd.prefs.components.conversation.labelling_box.enabled') ){
            this._conversationLabels.load(thread, true);
        }
    },


    /**
    * Notification that a thread was archived.
    * If code-requested via ajax, then move to next item in conversation
    * @function {} ?
    */
    _threadArchived: function(event){
        var conversationView = this._gtd.gmail.getCurrentViewUIObject();
        var thread = conversationView.getThread();	
        if( !event.environmentGenerated && thread.hasId(event.item) ){
            var elNext = conversationView.getMoveNextElement();
            if( elNext ) glUIWidget.simulateMouseEvent(elNext, 'click');
        }
    },

    /**
     * Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return false;
    },
    
    //}
});
