/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The popupable container for Threads Viewer and Thread Viewer.
 *
 * @class ? @extends gtdUIObject
 */

// TODO Give users option to, if in 'tl', load all left-clicked Popups as a ThreadList. That would be GTD Search, Dashboard, Label Expands. Should it be left/right click based (even on buttons?). 

var gtdExplorerPopup = gtdUIObject.extend({
    // Properties {
    
    
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	    	
    	/** @property {private gtdExplorerThreadsViewer} ? */
    	_threadsViewer: null,
    	
    	/** @property {private gtdExplorerThreadViewer} ?  */
    	_threadViewer: null,

	/** @property {private String} ? The method used to update the server */
	_serverActionType: null,
    	    	
	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	/** @property {private Element} - The el that contains the threads-viewer and thread-viewer */
	_elContainer: null,
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
	*/
	connect: function(gtd) {
		glLogger.log('Connecting gtdExplorerPopup object to view', 'gtdExplorerPopup');
		
		this._gtd = gtd;
		this._eventsContainer = new glEventsContainer(this);
		this.base(gtd);
		
		// Create container
		var doc = gtd.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.className = "gtdi gtdi-popup";
		this.el.style.position = "absolute";
		
		this._elContainer = doc.createElement("DIV");
		this._elContainer.id = "gtdi-dash-container";
		this.el.appendChild( this._elContainer );
		
		// Add the close:
		var elClose = doc.createElement("A");
		elClose.className = "gtdi-close";
		elClose.title = glI18N.getString("ExplorerPopup.close");
		elClose.innerHTML = glI18N.getString("ExplorerPopup.close");
		this._elContainer.appendChild( elClose );
		
		this.active(false);
		
		// Attach to view:
		this._gtd.gmail.canvasDocument.body.appendChild(this.el);
		
		// Add Event
		this._eventsContainer.observe(elClose, "click", this._clickClose, false);
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		glLogger.log('Disconnecting gtdExplorerPopup from view', 'gtdExplorerPopup');
		
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ){
			if( this[p] && this[p].nodeName && this[p].nodeType ){
				this[p] = null;
			}
		}
		
		if( this._threadsViewer ){
			this._threadsViewer.disconnect();
			this._threadsViewer = null;
		}
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		
		this.base();
	},
	
	
	/**
	* If doesn't exist, create it
	* Otherwise, return cached version
	* @function {gtdExplorerThreadsViewer} ?
	* @param {boolean} - If true, make visible
	* @param {optional int} x - The x position to open with
	* @param {optional int} y - The y position to open with
	* @return The cached explorerThreadsViewer
	*/
	getThreadsViewer: function(setActive, x, y){
		if( !this._threadsViewer ){
			this._threadsViewer = this._gtd.getUIObject("explorerThreadsViewer");
			this._threadsViewer.addEventListener(gtdExplorerThreadsViewer.CLICK_HEADER, this._clickHeader, this);
			this._threadsViewer.addEventListener(gtdExplorerThreadsViewer.LOAD_THREAD, this._loadThread, this);
			this._elContainer.appendChild( this._threadsViewer.el );
			this.setServerActionType(this._serverActionType);
		}
		if( this._threadViewer ) this._threadViewer.el.style.display = "none";
		this._threadsViewer.el.style.display = "";
		if( setActive ) this.active(true, x, y);
		return this._threadsViewer;
	},
	
	/**
	* If doesn't exist, create it
	* Otherwise, return cached version
	* @function {gtdExplorerThreadsViewer} ?
	* @param {boolean} - If true, make visible
	* @param {optional int} x - The x position to open with
	* @param {optional int} y - The y position to open with
	* @return The cached explorerThreadsViewer
	*/
	getThreadViewer: function(setActive, x, y, showHeader){
		if( !this._threadViewer ){
			this._threadViewer = this._gtd.getUIObject("explorerThreadViewer");
			this._threadViewer.addEventListener(gtdExplorerThreadViewer.CLICK_HEADER, this._clickHeader, this);
			this._threadViewer.addEventListener(gtdExplorerThreadViewer.CLICK_BACK, this._clickBack, this);
			this._threadViewer.addEventListener(gtdExplorerThreadViewer.CLICK_OPEN, this._clickClose, this);
			this._elContainer.appendChild( this._threadViewer.el );
			this.setServerActionType(this._serverActionType);
		}
		if( this._threadsViewer ) this._threadsViewer.el.style.display = "none";
		
		if( typeof showHeader=='undefined' ) showHeader = true;
		this._threadViewer.showHeader(showHeader);
		this._threadViewer.el.style.display = "";
		if( setActive ) this.active(true, x, y);
		return this._threadViewer;
	},
	
	/**
	* Notification that a header was clicked
	* Start drag
	* @function ?
	* @param {Object} event - The event from the viewer class. {name: , mouseEvent: Event}
	*/
	_clickHeader: function(event){
		// Record start point:
		this._drag = {pointerX:event.mouseEvent.pageX, pointerY: event.mouseEvent.pageY, left:parseInt(this.el.style.left), top:parseInt(this.el.style.top)};
		// Listen to mouse move:
		this._eventsContainer.observe(this._gtd.gmail.canvasDocument.body, 'mousemove', this._dragMouseMove, false);
		this._eventsContainer.observe(this._gtd.gmail.canvasDocument.body, 'mouseup', this._dragMouseUp, false);
	},
	
	/**
	* Calculate the offset/delta from the start point as the mouse move
	* @function ?
	*/
	_dragMouseMove: function(event){
		var delta = [event.pageX - this._drag.pointerX, event.pageY - this._drag.pointerY];
		this.el.style.left = (this._drag.left + delta[0]) + "px";
		this.el.style.top = (this._drag.top + delta[1]) + "px";
	},
	
	/**
	* Stop monitoring mouse move 
	* @function ?
	*/
	_dragMouseUp: function(event){
		this._eventsContainer.stopObserving(this._gtd.gmail.canvasDocument.body, 'mousemove', this._dragMouseMove, false);
		this._eventsContainer.stopObserving(this._gtd.gmail.canvasDocument.body, 'mouseup', this._dragMouseUp, false);
	},
	
	
	/**
	* Notification that a close was clicked
	* Hide the popup
	* @function ?
	*/
	_clickClose: function(event){
		// TODO notify classes like ExplorerThreadsViewer to stop responding to certain loading events? 
		try{
			this.active(false);
		}catch(e){
			glLogger.warn(e, 'gtdExplorerPopup._clickClose', this._gtd.gmail);
		}
	},	
	
	/**
	* Notification that a thread was loaded
	* Switch to message view
	* @function ?
	*/
	_loadThread: function(event){
		var threadViewer = this.getThreadViewer();
		threadViewer.load(this._threadsViewer.getViewerItem().getTitle(), event.thread, this._threadsViewer.getCurrentThreadListContainer().getThreadList().getFilteredThreads() );
	},
	
	/**
	* Notification that a user clicked 'back' on a thread
	* @function ?
	*/
	_clickBack: function(event){
		this.getThreadsViewer();
	},
	
	
	/**
	* Set popup visible/invisible
	* @function {Boolean} ?
	* @param {Boolean} active - If true, make visible.
	* @param {optional int} x - The x position to open with
	* @param {optional int} y - The y position to open with
	* @return Current on/off status
	*/
	active: function(active, x, y){
		if( typeof active!="undefined" ){
			if( this._gtd.gmail.accountData.getPref('plugins.gtd.prefs.general.centre_popup') ){ 
				// Centre popup:
				var popupDims = glUIWidget.getDimensions(this._elContainer);
				var viewport = glUIWidget.viewport(this.el.ownerDocument);
								
				x = (viewport[0]/2)-(popupDims.width/2);
				y = (viewport[1]/2)-(popupDims.height/2) + this.el.ownerDocument.defaultView.pageYOffset;
				
				var xy = this.boundsCheckXY(x,y,true,true);
				x = xy[0];
				y = xy[1];
			}
			
			this.el.style.display = active? "" : "none";
			if( typeof x!="undefined" ) this.el.style.left = x + "px";
			if( typeof y!="undefined" ) this.el.style.top = y + "px";
			
			if( active ){
				this._gtd.gmail.lightbox.show();
				/*
				setTimeout( glBase.bind(this, function(){
					this._eventsContainer.observe(this.el.ownerDocument.body, 'mousedown', this._clickWindow, false);
				}), 50 );
				*/
			}else{
				this._gtd.gmail.lightbox.hide();
				//this._eventsContainer.stopObserving(this.el.ownerDocument.body, 'mousedown', this._clickWindow, false);
				this.dispatchEvent({name: gtdExplorerPopup.HIDE});
			}
		}
		return (this.el.style.display=="")
	},
	
	/**
	* Set method for updating server. Ensure future created objects can use this info; and existing objects are updated.
	* @function ?
	* @param {String} type - The type. A static variable of glActionManager
	*/
	setServerActionType: function(type){
		this._serverActionType = type;
		if( this._threadViewer ) this._threadViewer.setServerActionType(type);
		if( this._threadsViewer ) this._threadsViewer.setServerActionType(type);
	},
	
	/**
	* Notification that the window was clicked. If active, and not on el, hide.
	* @function {} ?
	*/
	_clickWindow: function(event){
		try{
			// Ensure whatever clicked not in line
			var el = event.target;
			while( el.parentNode ){
				if( el==this.el ) return;
				el = el.parentNode;
			}
			this.active(false);
		}catch(e){
			glLogger.warn(e, 'gtdExplorerPopup._clickWindow', this._gtd.gmail);
		}
	},
	
	/**
	* Get x,y coordinates from an event; taking into account the size of our own .el and the requested location.
	* @function {int[]} ?
	* @param {Event} event - The DOM event
	* @param {String} location - The desired location
	* @param {Boolean} scrollToView - If true, move window to encapsulate x,y and .el
	*/
	getXYFromEvent: function(event, location, scrollToView){
		x = event.pageX;
		y = event.pageY;
		
		var popupDims = glUIWidget.getDimensions(this._elContainer);
		var viewport = glUIWidget.viewport(this.el.ownerDocument);
		switch(location){
			case "left":
				x = x - popupDims.width;
				break;
			case "bottomLeft":
				x = x - popupDims.width;
				break;
		}
				
		if( scrollToView ){
			if( (y+popupDims.height) > viewport[1] ){
				// TODO ensure we don't go lower that the top of the el
				var d = (y + popupDims.height) - viewport[1];
				//this.el.ownerDocument.defaultView.scrollTo(0, d);
			}
		}
		
		return [x,y];
	},
	
	/**
	* Get x,y coordinates from an element; taking into account the size of our own .el and the requested location.
	* @function {int[]} ?
	* @param {Element} el - The element
	* @param {String} location - The desired location
	* @param {Boolean} scrollToView - If true, move window to encapsulate x,y and .el
	*/
	getXYFromElement: function(el, location, scrollToView){
		if( el.nodeName=="A" ) el = el.parentNode;	// Cannot reference from A
	
		var x = 0;
		var y = 0;
		var borderWidth = el.parentNode? glUIWidget.getBorderWidth(el.parentNode) : 0;	// get width of el's parent (canvas?) border, which is not included in the cumulativeOffset
		var offset = glUIWidget.cumulativeOffset(el);
			offset[0] = offset[0] + borderWidth;
			offset[1] = offset[1] + borderWidth;
		var dims = glUIWidget.getDimensions(el);
		var popupDims = glUIWidget.getDimensions(this._elContainer);
		var viewport = glUIWidget.viewport(this._elContainer.ownerDocument);
		

		switch(location){
			case "right":
				x = offset[0] + dims.width;
				y = offset[1];
				break;
			case "left":
				x = offset[0] - dims.width - popupDims.width;
				y = offset[1];
				break;
			case "bottomRight":
				x = offset[0] + dims.width;
				y = offset[1] + dims.height;
				break;
			case "bottomLeft":
				x = offset[0] - dims.width  - popupDims.width;
				y = offset[1] + dims.height;
				break;
			case "bottom":
				x = offset[0];
				y = offset[1] + dims.height;
				break;
			case "relative-bottom":
				x = 0;
				y = dims.height;
				break;
			case "relative-bottom-bottom-right":
				x = dims.width - popupDims.width;
				y = dims.height;
				break;
		}
		
		if( scrollToView ){
			if( (y+popupDims.height) > viewport[1] ){
				// TODO ensure we don't go lower that the top of the el
				var d = (y + popupDims.height) - viewport[1];
				//this.el.ownerDocument.defaultView.scrollTo(0, d);
			}
		}
		
		
		return [x,y];
	},
	
	/**
	* Check we're in bounds with the proposed x/y
	* @function {int[]} ?
	* @param {optional int} x - The x position 
	* @param {optional int} y - The y position 
	* @return Array of potentially modified x/y
	*/
	boundsCheckXY: function(x,y, ignoreX, ignoreY){
		
		// Check right/bottom
		var popupDims = glUIWidget.getDimensions(this._elContainer);
		var viewport = glUIWidget.viewport(this._elContainer.ownerDocument);
		
		
		if( !ignoreX ) if( (x + popupDims.width) > viewport[0] ) x = viewport[0] - popupDims.width - 30;
		if( !ignoreY ) if( (y + popupDims.height) > viewport[1] ) y = viewport[1] - popupDims.height - 30;
		
		
		// Check not off left/top
		if( x<0 ) x = 0;
		if( y<0 ) y = 0;
		
		return [x,y];
		
	},
	
	
	/**
	* Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
	* @function {Boolean} ?
	* @return Whether the UI can be cached
	*/
	isCacheable: function() {
		return true;
	},
	

},{
	// Static Properties:
	HIDE: 'hide',
});
gtdExplorerPopup.implement(glEventDispatcher); // Add event dispatching support