/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The UI for the current item (and changing it) in gtdExplorerThreadsViewer
 *
 * @class ? @extends gtdUIObject
 */
var gtdExplorerThreadsItem = gtdUIObject.extend({
    // Properties {
    
    
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
	 
    
	/**
	* The thread list container
	* @property {Element} ?
	*/
	el: null,
	
	/** @property {private Element} ?  */
	_elType: null,
	
	/** @property {private Element} ?  */
	_elItem: null,
	
	
	
	
    //}


	// Methods {
	/**
	* Connects the element to the given gtdMainPagePlugin object's page. Called
	* automatically by {@link gtdMainPagePlugin.getUIObject}.
	* @function ?
	* @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
	*/
	connect: function(gtd) {
		glLogger.log('Connecting gtdExplorerThreadsItem object to view', 'gtdExplorerThreadsItem');
		
		this._gtd = gtd;
		this._eventsContainer = new glEventsContainer(this);
		this.base(gtd);
		
		// Create container
		var doc = gtd.gmail.canvasDocument;
		this.el = doc.createElement("DIV");
		this.el.id = "gtdi-context";
		
		this._elType = doc.createElement("H1");
		this.el.appendChild( this._elType );
		
		this._elItem = doc.createElement("SPAN");
		this.el.appendChild( this._elItem );
		
		
	},



	/**
	* Called on page shutdown, it does cleanup
	* @function ?
	*/
	disconnect: function() {
		glLogger.log('Disconnecting gtdExplorerThreadsItem from view', 'gtdExplorerThreadsItem');
		
		if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
		this.el = null;
		
		// Wipe references to elements
		for( p in this ){
			if( this[p] && this[p].nodeName && this[p].nodeType ){
				this[p] = null;
			}
		}
		
		this._eventsContainer.reset();
		this._eventsContainer = null;
		
		this.base();
	},
	
	
	/**
	* Set the titles
	* @function ?
	* @param {String} type - The type of item, as defined in the static properties of gtdExplorerThreadsViewer
	* @param {String} item - The name of the item (e.g. a label or a contact email)
	*/
	setType: function(type, item){
		this._elType.innerHTML = glI18N.getString("ExplorerThreadsItem." + type, type) + (item? ":" : "");
		this._elItem.innerHTML = item;
	},
	
	/**
	* Get the title of the current view
	* @function {String} ?
	*/
	getTitle: function(){
		return this._elType.innerHTML + " " + this._elItem.innerHTML;
	},
		
	/**
	* Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
	* @function {Boolean} ?
	* @return Whether the UI can be cached
	*/
	isCacheable: function() {
		return false;
	},
	

});
gtdExplorerThreadsItem.implement(glEventDispatcher); // Add event dispatching support