/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * A button next to the standard Gmail search button that loads the async results
 *
 * @class ? @extends gtdUIObject
 */
var gtdSearchButton = gtdUIObject.extend({
    // Properties {
    /**
     * The search button container
     * @property {Element} ?
     */
    el: null,
    
	/** @property {private gtdMainPagePlugin} ? The main gtd plugin for this page */
	_gtd: null,
	
	/** @property {private glEventsContainer} ? Collection of manageable events on DOM elements */
	_eventsContainer: null,
    //}

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        glLogger.log('Connecting gtdSearchButton object to view', 'gtdSearchButton');

	this._gtd = gtd;
        this.base(gtd);
	this._eventsContainer = new glEventsContainer(this);

        var searchBox = gtd.gmail.getUIObject('searchBox');

        // Build Node {
        this.el = glUIWidget.createGmailButton("gmail-header-button", glI18N.getString("Search.Button"), null, searchBox.searchBox.ownerDocument);
        //}

	this._eventsContainer.observe(this.el, "click", this._click, false);
		this._eventsContainer.observe(this.el, "keydown", this._key, false);

        // Insert into document {
        searchBox.searchMailButton.parentNode.insertBefore(this.el, searchBox.searchMailButton.nextSibling);
        //}
    },

	/**
	* Respond to a key press. If enter/space, go to click functionality
	* @function ?
	*/
	_key: function(event){
		if( event.keyCode==13 || event.keyCode==32 ){
			this._click(event);
		}
	},

	/** Respond to a search button click -> extract current query and open in async popup
	* @function ?
	*/
	_click: function(event){
		try{
			var searchBox = this._gtd.gmail.getUIObject('searchBox');
			if( searchBox.searchBox.value=="" ) return;
		
		
			// Get the popup
			var explorerPopup = this._gtd.getUIObject("explorerPopup");
		
			var threadsViewer = explorerPopup.getThreadsViewer(true);

			// Get the x/y below the Dashboard button:
			var xy = explorerPopup.getXYFromElement(searchBox.searchBox, "bottom", true);
			xy = explorerPopup.boundsCheckXY(xy[0], xy[1], null, true);
			explorerPopup.setServerActionType(glActionManager.USE_AJAX);
			explorerPopup.active(true, xy[0], xy[1]);
			// Load the Dashboard into the ThreadsViewer
			threadsViewer.load( gtdExplorerThreadsViewer.SEARCH, searchBox.searchBox.value );
		
		}catch(e){
			glLogger.warn(e, 'gtdSearchButton._click', this._gtd.gmail);
		}
	},

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        glLogger.log('Disconnecting gtdSearchFiltersDropdown from view', 'gtdSearchFiltersDropdown');

        if( this.el && this.el.parentNode ) this.el.parentNode.removeChild(this.el);
        this.el = null;
	this._eventsContainer.reset();
	
        this.base();
    },
    //}
});