/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Andy Mitchell (andy@gtdinbox.com)
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * 
 * ***** END LICENSE BLOCK ***** */


/**
 * The base class for all GTD UI objects.
 *
 * @class {abstract} gtdUIObject
 */
var gtdUIObject = glBase.extend({
    // Properties {
    /**
     * Whether or not it is connected. Useful for only running connect once or
     * re-connecting when it gets disconnected.
     * @property {Boolean} ?
     */
    _isConnected: null,
    //}

    /**
     * The constructor.
     * @constructor ?
     */
    constructor: function() {
        if(this.constructor === gtdUIObject) throw new Error('Cannot be instantiated');
    },

    // Methods {
    /**
     * Connects the element to the given gtdMainPagePlugin object's page. Called
     * automatically by {@link gtdMainPagePlugin.getUIObject}.
     * @function ?
     * @param {gtdMainPagePlugin} gtd - The gtdMainPagePlugin object for the page
     */
    connect: function(gtd) {
        this._isConnected = true;
    },

    /**
     * Called on page shutdown, it does cleanup
     * @function ?
     */
    disconnect: function() {
        this._isConnected = false;
    },

    /**
     * Whether the UI object can be cached by {@link gtdMainPagePlugin.getUIObject}
     * @function {Boolean} ?
     * @return Whether the UI can be cached
     */
    isCacheable: function() {
        return true;
    },

    /**
     * Returns whether or not connected
     * @function {Boolean} ?
     * @return Connection status
     */
    isConnected: function() {
        return this._isConnected;
    },
    //}
});