/*
 * ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is GTDInbox.
 *
 * The Initial Developer of the Original Code is
 * Stephen Augenstein
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Andy Mitchell (andy@gtdinbox.com)
 * 
 * ***** END LICENSE BLOCK ***** */



/*
	Base.js, version 1.1
	Copyright 2006-2007, Dean Edwards
    gl prefix to protect global namespace added by Stephen Augenstein - 2008
	License: http://www.opensource.org/licenses/mit-license.php
*/

var glBase = function() {
	// dummy
};

glBase.extend = function(_instance, _static) { // subclass
	var extend = glBase.prototype.extend;
	
	// build the prototype
	glBase._prototyping = true;
	var proto = new this;
	extend.call(proto, _instance);
	delete glBase._prototyping;
	
	// create the wrapper for the constructor function
	//var constructor = proto.constructor.valueOf(); //-dean
	var constructor = proto.constructor;
	var klass = proto.constructor = function() {
		if (!glBase._prototyping) {
			if (this._constructing || this.constructor == klass) { // instantiation
				this._constructing = true;
				constructor.apply(this, arguments);
				delete this._constructing;
			} else if (arguments[0] != null) { // casting
				return (arguments[0].extend || extend).call(arguments[0], proto);
			}
		}
	};
	
	// build the class interface
	klass.ancestor = this;
	klass.extend = this.extend;
	klass.forEach = this.forEach;
	klass.implement = this.implement;
	klass.prototype = proto;
	klass.toString = this.toString;
	klass.valueOf = function(type) {
		//return (type == "object") ? klass : constructor; //-dean
		return (type == "object") ? klass : constructor.valueOf();
	};
	extend.call(klass, _static);
	// class initialisation
	if (typeof klass.init == "function") klass.init();
	return klass;
};

glBase.prototype = {	
	extend: function(source, value) {
		if (arguments.length > 1) { // extending with a name/value pair
			var ancestor = this[source];
			if (ancestor && (typeof value == "function") && // overriding a method?
				// the valueOf() comparison is to avoid circular references
				(!ancestor.valueOf || ancestor.valueOf() != value.valueOf()) &&
				/\bbase\b/.test(value)) {
				// get the underlying method
				var method = value.valueOf();
				// override
				value = function() {
					var previous = this.base || glBase.prototype.base;
					this.base = ancestor;
					var returnValue = method.apply(this, arguments);
					this.base = previous;
					return returnValue;
				};
				// point to the underlying method
				value.valueOf = function(type) {
					return (type == "object") ? value : method;
				};
				value.toString = glBase.toString;
			}
			this[source] = value;
		} else if (source) { // extending with an object literal
			var extend = glBase.prototype.extend;
			// if this object has a customised extend method then use it
			if (!glBase._prototyping && typeof this != "function") {
				extend = this.extend || extend;
			}
			var proto = {toSource: null};
			// do the "toString" and other methods manually
			var hidden = ["constructor", "toString", "valueOf"];
			// if we are prototyping then include the constructor
			var i = glBase._prototyping ? 0 : 1;
			while (key = hidden[i++]) {
				if (source[key] != proto[key]) {
					extend.call(this, key, source[key]);

				}
			}
			// copy each of the source object's properties to this object
			for (var key in source) {
				if (!proto[key]) extend.call(this, key, source[key]);
			}
		}
		return this;
	},

	base: function() {
		// call this method from any other method to invoke that method's ancestor
	}
};

// initialise
glBase = glBase.extend({
	constructor: function() {
		this.extend(arguments[0]);
	}
}, {
	ancestor: Object,
	version: "1.1",
	
	forEach: function(object, block, context) {
		for (var key in object) {
			if (this.prototype[key] === undefined) {
				block.call(context, object[key], key, object);
			}
		}
	},
		
	implement: function() {
		for (var i = 0; i < arguments.length; i++) {
			if (typeof arguments[i] == "function") {
				// if it's a function, call it
				arguments[i](this.prototype);
			} else {
				// add the interface using the extend method
				this.prototype.extend(arguments[i]);
			}
		}
		return this;
	},
	
	toString: function() {
		return String(this.valueOf());
	},
	
	
	
	// TODO potentially find more relevant homes for these static functions:
	
	decodeEntities: function(s){
		if( !s ) return "";
		s = s.replace(/&amp;/g,'&').replace(/&lt;/g,'<').replace(/&gt;/g,'>').replace(/&quot;/g, '"').replace(/&apos;/g, "'").replace(/&hellip;/g, String.fromCharCode(133));
		var m = s.match(/&#(\d+);/g);
		if( m ){
			for( var i = 0; i < m.length; i++ ){
				var m2 = m[i].match(/&#(\d+);/);
				s = s.replace(m2[0], String.fromCharCode(m2[1]));
			}
		}
		return s;
	},
	
	realEmail: function(email){
		if( email && email.replace ){
			return email.replace(/\+([^ \.]*?)@/g, "@");
		}
	},
	
	testEmailAddress: function(e, allowEnclosingQuotes){
            if( allowEnclosingQuotes ){
		return /^\"?([a-zA-Z0-9])+([\.a-zA-Z0-9\+_-])*@([a-zA-Z0-9])+(\.[a-zA-Z0-9_-]+)+\"?$/.test(e);
            }else{
                return /^([a-zA-Z0-9])+([\.a-zA-Z0-9\+_-])*@([a-zA-Z0-9])+(\.[a-zA-Z0-9_-]+)+$/.test(e);
            }
	},
	
	stripTags: function(s){
		return s? s.replace(/\<[^>]+?\>/g, "") : "";
	},
	
	prepRE: function(s){
		return s.replace(/([\"\'\*\=\|\!\:\{\}\\\.\+\-\?\&\[\]\^\$])/g, "\\$1");
	},
	
	bind: function(object, method) {			
		return function () {
			return method.apply(object, arguments);
		};
	},
	
	hashLength: function(h){
		var c = 0;
		for( i in h ) c++;
		return c;
	},
	
	hashKeysToArray: function(h){
		var a = [];
		for( i in h ) a.push(i);
		return a;
	},
	
	arrayToHash: function(a, forceLowerCase, h){
		h = h || {};
		for( var i = 0; i < a.length; i++ ){
			if( forceLowerCase ) a[i] = a[i].toLowerCase();
			h[a[i]] = true;
		}
		return h;
	},
	
	hashToArray: function(h){
		var a = [];
		for( k in h ) a.push( h[k] );
		return a;
	},
	
	stopEvent: function(event) {
		if (event.preventDefault) {
			event.preventDefault();
			event.stopPropagation();
		} else {
			event.returnValue = false;
			event.cancelBubble = true;
		}
	},
	
});
